/**************************************************************************//**
 * @file
 * @brief VMON Demo
 * @version  1.10

 ******************************************************************************
 * @section License
 * <b>(C) Copyright 2017 Silicon Labs, http://www.silabs.com</b>
 *******************************************************************************
 *
 * This file is licensed under the Silabs License Agreement. See the file
 * "Silabs_License_Agreement.txt" for details. Before using this software for
 * any purpose, you must agree to the terms of that agreement.
 *
 ******************************************************************************/
#include "em_device.h"
#include "em_chip.h"
#include "em_emu.h"
#include "em_cmu.h"
#include <stdio.h>
#include "display.h"
#include "textdisplay.h"
#include "retargettextdisplay.h"

/* VMON rail voltage thresholds, in millivolts */
#define VMON_AVDD_RISE_THRESHOLD_mV  3000
#define VMON_AVDD_FALL_THRESHOLD_mV  2900
#define VMON_DVDD_THRESHOLD_mV       2800
#define VMON_IOVDD_THRESHOLD_mV      2700

/* Flags to keep the state of the power rails */
volatile bool AVDD_good = true;
volatile bool DVDD_good = true;
volatile bool IOVDD_good = true;

/***************************************************************************//**
 * @brief
 *   EMU IRQ handler. Handles VMON interrupts for this example.
 ******************************************************************************/
void EMU_IRQHandler(void)
{
  uint32_t flags = EMU_IntGet();
  EMU_IntClear(flags);

  if (flags & EMU_IF_VMONAVDDFALL) {
    AVDD_good = false;
  } else if (flags & EMU_IF_VMONAVDDRISE) {
    AVDD_good = true;
  }

  if (flags & EMU_IF_VMONDVDDFALL) {
    DVDD_good = false;
  } else if (flags & EMU_IF_VMONDVDDRISE) {
    DVDD_good = true;
  }

  if (flags & EMU_IF_VMONIO0FALL) {
    IOVDD_good = false;
  } else if (flags & EMU_IF_VMONIO0RISE) {
    IOVDD_good = true;
  }
}

/**************************************************************************//**
 * @brief Setup VMON to monitor AVDD, DVDD, and IOVDD
 *****************************************************************************/
void VMON_Init(void)
{
  /* Initialize AVDD thresholds */
  EMU_VmonHystInit_TypeDef avddInit = EMU_VMONHYSTINIT_DEFAULT;
  avddInit.channel = emuVmonChannel_AVDD;
  avddInit.riseThreshold = VMON_AVDD_RISE_THRESHOLD_mV;
  avddInit.fallThreshold = VMON_AVDD_FALL_THRESHOLD_mV;
  EMU_VmonHystInit(&avddInit);

  /* Initialize DVDD threshold */
  EMU_VmonInit_TypeDef dvddInit = EMU_VMONINIT_DEFAULT;
  dvddInit.channel = emuVmonChannel_DVDD;
  dvddInit.threshold = VMON_DVDD_THRESHOLD_mV;
  EMU_VmonInit(&dvddInit);

  /* Initialize IOVDD threshold */
  EMU_VmonInit_TypeDef iovddInit = EMU_VMONINIT_DEFAULT;
  iovddInit.channel = emuVmonChannel_IOVDD0;
  iovddInit.threshold = VMON_IOVDD_THRESHOLD_mV;
  EMU_VmonInit(&iovddInit);

  /* Enable all VMON interrupts */
  uint32_t flags = EMU_IEN_VMONAVDDFALL
                 | EMU_IEN_VMONAVDDRISE
                 | EMU_IEN_VMONIO0FALL
                 | EMU_IEN_VMONIO0RISE
                 | EMU_IEN_VMONDVDDFALL
                 | EMU_IEN_VMONDVDDRISE;
  EMU_IntEnable(flags);
  NVIC_EnableIRQ(EMU_IRQn);
}

/**************************************************************************//**
 * @brief Print power rail status to the LCD
 *****************************************************************************/
void printVoltageStatus(void)
{
  printf("\f"); // Clear the LCD screen
  printf("\n      VMON DEMO\n\n");
  if (AVDD_good) {
    printf(" AVDD: ROSE ABOVE \n       %dmV Thresh\n\n", VMON_AVDD_RISE_THRESHOLD_mV);
  }else {
    printf(" AVDD: FELL BELOW \n       %dmV Thresh\n\n", VMON_AVDD_FALL_THRESHOLD_mV);
  }

  if (DVDD_good) {
    printf(" DVDD: ROSE ABOVE \n       %dmV Thresh\n\n", VMON_DVDD_THRESHOLD_mV);
  }else {
    printf(" DVDD: FELL BELOW \n       %dmV Thresh\n\n", VMON_DVDD_THRESHOLD_mV);
  }

  if (IOVDD_good) {
    printf("IOVDD: ROSE ABOVE \n       %dmV Thresh\n\n", VMON_IOVDD_THRESHOLD_mV);
  }else {
    printf("IOVDD: FELL BELOW \n       %dmV Thresh\n\n", VMON_IOVDD_THRESHOLD_mV);
  }
}

/**************************************************************************//**
 * @brief  Main function
 *
 * This example demonstrates the usage of the Voltage Monitor (VMON) in the
 * EMU on Series 1 devices.
 *
 * The VMON is enabled and monitors the AVDD, DVDD, and IOVDD rails and
 * displays their status to the STK's memory LCD. The thresholds for each
 * rail are set by the #defines at the top of this file:
 *  - VMON_AVDD_RISE_THRESHOLD_mV
 *  - VMON_AVDD_FALL_THRESHOLD_mV
 *  - VMON_DVDD_THRESHOLD_mV
 *  - VMON_IOVDD_THRESHOLD_mV
 *
 *  Note: Only the AVDD rail has separate rising and falling thresholds.
 *
 *  When a voltage threshold is crossed, the EMU generates an interrupt which
 *  is handled by the EMU_IRQHandler function. This sets flags to indicate
 *  whether a voltage rail is 'good' (above its threshold) or 'bad' (below
 *  its threshold). In the main while loop, this status is then printed to
 *  the LCD, and the device enters EM3 to wait for the next interrupt.
 *
 *  To test:
 *    1) Place the STK power switch into BAT to disconnect the debugger's
 *       power supply.
 *    2) Attach a power supply to GND and VMCU (pins 1 and 2 on the expansion
 *       header)
 *    3) Vary the power supply voltage above/below the defined thresholds. The
 *       device will update the status on the LCD when a voltage crosses a
 *       threshold.
 *
 *****************************************************************************/
int main(void)
{
  /* Chip errata */
  CHIP_Init();

  /* Bypass DCDC so that DVDD is powered from VMCU */
#if defined(_EMU_DCDCCTRL_MASK)
  EMU_DCDCInit_TypeDef dcdcInit = EMU_DCDCINIT_DEFAULT;
  dcdcInit.dcdcMode = EMU_DCDCCTRL_DCDCMODE_BYPASS;
  EMU_DCDCInit(&dcdcInit);
#endif

  /* Set the HFRCO Band to 38 MHz for quicker LCD refresh */
  CMU_HFRCOBandSet(cmuHFRCOFreq_38M0Hz);

  /* Initialize the STK's LCD */
  DISPLAY_Init();

  /* Retarget stdio to a text display. */
  if (RETARGET_TextDisplayInit() != TEXTDISPLAY_EMSTATUS_OK)
  {
    /* Text display initialization failed. */
    while (1);
  }

  /* Initialize and enable the VMON for AVDD, DVDD, IOVDD */
  VMON_Init();

  while (1)
  {
    /* Print the current rail status to the LCD */
    printVoltageStatus();

    /* Enter EM3 until the next interrupt occurs */
    EMU_EnterEM3(true);
  }
}
